local typo = {}
local stack = {}

local first = true

local font = love.graphics.newFont("fnt/TTWPGOTT.ttf", 32)
love.graphics.setFont(font)

local nnum = 1 -- for sounds
local currentlySel = 2 -- current menu item 
local doneWithAll = false -- done with into
local isPlaying = false

local currentGroup = "mainMenu" -- current menu

local function chooseSound(s)
	if s == "" then
		return "linebreak"
	else
		return "rollup"..nnum
	end
end

local function doneFun(three, s)
	local t = three or false

	for i,v in ipairs(typo[currentGroup]) do
		if t then
			flux.to(v, durationSound("rollup1"), {y = v.y - font:getHeight()}):delay(v.maxT):onstart( function() playSound("rollup1") end ):oncomplete( function()
				flux.to(v, durationSound("rollup2"), {y = v.y - font:getHeight()}):delay(v.maxT):onstart( function() playSound("rollup2") end ):oncomplete( function()
					flux.to(v, durationSound("rollup3"), {y = v.y - font:getHeight()}):delay(v.maxT):onstart( function() playSound("rollup3") end ):oncomplete( function() pushStack() end ) end ) end )
		else
			if i == #typo[currentGroup] then
				flux.to(v, durationSound(chooseSound(s)), {y = v.y - font:getHeight()}):delay(v.maxT):onstart( function() playSound(chooseSound(s)) end ):oncomplete( function() pushStack() end )
				nnum = (nnum + 1) % 4
				if nnum == 0 then nnum = 1 end
			else
				flux.to(v, durationSound(chooseSound(s)), {y = v.y - font:getHeight()}):delay(v.maxT)
			end
		end
	end
end

local function rollBack()
	if #stack[currentGroup] == 0 then
		for i,v in ipairs(typo[currentGroup]) do
			if i == #typo[currentGroup] then
				flux.to(v, durationSound("rollup"..nnum), {y = v.y + h}):delay(v.maxT):onstart( function() playSound("rollup"..nnum) end )
				nnum = (nnum + 1) % 4
				if nnum == 0 then nnum = 1 end
			else
				flux.to(v, durationSound("rollup"..nnum), {y = v.y + h}):delay(v.maxT)
			end

			if v.sel then
				v.obj:set(v.s)
				v.w = v.w - font:getWidth(">") - font:getWidth("<")
				v.sel = false
			end
		end
	end

	if currentGroup == "start" then gameState = "game" end
end

function setMenu(group)
	currentGroup = group
end

function newText(s, x, y, timer, group, endGroup, centred, start)
	local c = centred or false
	local ss = start or false

	local i = {
		obj = love.graphics.newText(love.graphics.getFont(), ""),
		s = s,
		i = 1, -- current char index
		
		maxT = timer, -- typing delays
		curT = timer,

		x = x,
		y = y,
		centred = c,
		g = group,
		eg = endGroup,

		done = false,

		w = 0,
		h = font:getHeight(),
		sel = false,
	}

	if type(stack[group]) ~= "table" then stack[group] = {} end
	
	table.insert(stack[group], i)

	if ss then
		pushStack()
	end

	return #stack[group]
end

function pushStack()
	if type(typo[currentGroup]) ~= "table" then typo[currentGroup] = {} end

	if currentGroup == "quit" then love.event.push("quit") end

	if stack[currentGroup][1] then
		table.insert(typo[currentGroup], stack[currentGroup][1])
		table.remove(stack[currentGroup], 1)
	end

	if typo[currentGroup][#typo[currentGroup]].done and #stack[currentGroup] == 0 then
		doneWithAll = true

		if not isPlaying then
			playSound("space"); playSound("ievanPolkka")
			isPlaying = true
		end
	end
end

function moveText(i, x, y)
	typo[currentGroup][i].x = x; typo[currentGroup][i].y = y
end

function updateText(dt)
	local v = typo[currentGroup][#typo[currentGroup]]
	if not v.done then
		v.curT = v.curT + dt

		if v.curT >= v.maxT then
			local s = v.s:sub(v.i, v.i)
			v.obj:add(s, v.w, 0)
			v.w = v.w + v.obj:getWidth()

			v.i = v.i + 1
			v.curT = 0

			if s == " " then
				playSound("space")
			elseif s == "" then
				--
			else
				playSound("key1")
			end

			if v.i > #v.s then
				v.done = true

				doneFun(first, s)
				if first then first = false end
			end
		end
	end

	if doneWithAll then
		if currentlySel > 0 then
			local v = typo[currentGroup][currentlySel]

			if currentGroup == "mainMenu" and not v.sel then
				v.obj:set(">"..v.s.."<")
				v.w = v.w + font:getWidth(">") + font:getWidth("<")
				v.sel = true
			end
		end
	end	
end

function keyText(key)
	if doneWithAll then
		if currentGroup == "mainMenu" and (key == "up" or key == "w") then
			playSound("key1")

			currentlySel = math.min(math.max(2, currentlySel), 6)
			local v = typo[currentGroup][currentlySel]
			v.obj:set(v.s)
			v.w = v.w - font:getWidth(">") - font:getWidth("<")
			v.sel = false

			currentlySel = currentlySel - 1
		elseif currentGroup == "mainMenu" and (key == "down" or key == "s") then
			playSound("key1")

			currentlySel = math.min(math.max(2, currentlySel), 6)
			local v = typo[currentGroup][currentlySel]
			v.obj:set(v.s)
			v.w = v.w - font:getWidth(">") - font:getWidth("<")
			v.sel = false

			currentlySel = currentlySel + 1
		end

		if key == "space" then
			for i,v in ipairs(typo[currentGroup]) do
				if i == #typo[currentGroup] then
					flux.to(v, durationSound("paperout"), {y = v.y - h}):onstart( function() playSound("paperout") end ):oncomplete( function() currentGroup = typo[currentGroup][currentlySel].eg; currentlySel = 2; pushStack(); rollBack() end )
				else
					flux.to(v, durationSound("paperout"), {y = v.y - h})
				end
			end
			doneWithAll = false
		end

		currentlySel = math.min(math.max(2, currentlySel), 6)
	else
		for i,v in ipairs(typo[currentGroup]) do
			v.maxT = 0.125
		end
		for i,v in ipairs(stack[currentGroup]) do
			v.maxT = 0.125
		end
	end
end

function drawText()
	for i,v in ipairs(typo[currentGroup]) do
		if v.centred then
			love.graphics.draw(v.obj, v.x - v.w/2, v.y - v.h/2)
		else
			love.graphics.draw(v.obj, v.x, v.y)
		end
	end
end