local state = {}

local mask_shader = love.graphics.newShader[[
   vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      if (Texel(texture, texture_coords).rgb == vec3(0.0)) {
         // a discarded pixel wont be applied as the stencil.
         discard;
      }
      return vec4(1.0);
   }
]]

local function ballStencil()
	love.graphics.setShader(mask_shader)
   	love.graphics.draw(player.mask, player.x, player.y)
   	love.graphics.setShader()
end

function state:load()
	player = {
		x = 0,
		y = 0,
		speed = 64,
		t = 0.25,

		body = love.graphics.newImage("img/body.png"),
		eyes = love.graphics.newImage("img/eyes.png"),
		mask = love.graphics.newImage("img/mask.png"),

		eyePos = 0,
		canMove = true,

		dir = "s",

		keys = { -- Keys down
			["w"] = false,
			["s"] = false,
			["a"] = false,
			["d"] = false,
		}
	}
end

function state:update(dt)
	if love.keyboard.isDown("w") and player.dir == "w" then player.y = player.y - player.speed * dt end
	if love.keyboard.isDown("s") and player.dir == "s" then player.y = player.y + player.speed * dt end
	if love.keyboard.isDown("a") and player.dir == "a" then player.x = player.x - player.speed * dt end
	if love.keyboard.isDown("d") and player.dir == "d" then player.x = player.x + player.speed * dt end

	for i,v in pairs(player.keys) do
		local key = ""
		if player.keys[i] then key = i end
		
		if player.canMove then
			if key == "w" and player.dir ~= "w" then
				if player.dir == "a" or player.dir == "s" then
					flux.to(player, player.t, {eyePos = -26}):onstart( function() player.canMove = false; end ):oncomplete( function() player.canMove = true; player.dir = "w" end )
				elseif player.dir == "d" then
					flux.to(player, player.t, {eyePos = 26}):onstart( function() player.canMove = false; end ):oncomplete( function() player.canMove = true; player.dir = "w" end )
				end
			end

			if key == "s" and player.dir ~= "s" then
				if player.dir == "w" then player.eyePos = 26 end
				flux.to(player, player.t, {eyePos = 0}):onstart( function() player.canMove = false; end ):oncomplete( function() player.canMove = true; player.dir = "s" end )
			end

			if key == "a" and player.dir ~= "a" then
				if player.dir == "w" then player.eyePos = -26 end
				flux.to(player, player.t, {eyePos = -13}):onstart( function() player.canMove = false; end ):oncomplete( function() player.canMove = true; player.dir = "a" end )
			end

			if key == "d" and player.dir ~= "d" then
				if player.dir == "w" then player.eyePos = 26 end
				flux.to(player, player.t, {eyePos = 13}):onstart( function() player.canMove = false; end ):oncomplete( function() player.canMove = true; player.dir = "d" end )
			end
		end
	end
end

function state:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(player.body, player.x, player.y)

	love.graphics.stencil(ballStencil, "replace", 1)
    love.graphics.setStencilTest("greater", 0)
    love.graphics.draw(player.eyes, player.x + player.eyePos, player.y)
    love.graphics.setStencilTest()
end

function state:key(key)
	if key == "a" or key == "d" or key == "w" or key == "s" then player.keys[key] = true end
end

function state:unkey(key)
	if key == "a" or key == "d" or key == "w" or key == "s" then player.keys[key] = false end
end

return state